unit ftmExceptionForm;
{------------------------------------------------------------------------------
   PROJECT NAME: KT2000 Common Units
   $Id: ftmExceptionForm.pas,v 1.10 2004/03/17 09:40:19 Kirys Exp $
 ------------------------------------------------------------------------------
   Universal Exception manager module.
   Self contained Exception manager unit with log generator
 ------------------------------------------------------------------------------
   Copyright (C) 2000-2003, http://www.kt2k.com
   All rights Reserved !
 ------------------------------------------------------------------------------
Internationalized & localized for HFS ~ HTTP File Server by SilentPliz - 2011-2019  <silentpliz@gmail.com>
 ------------------------------------------------------------------------------}

interface
{.$DEFINE FullDebug}
{.$DEFINE MODULES_LIST}
uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls,clipbrd,JclSysInfo,JclPeImage, gnugettext;

resourcestring
  RsAppError = '%s - Erreur d''application';
  RsExceptionClass = 'Classe d''exception : %s';
  RsExceptionAddr = 'Adresse de l''exception : %p';
  RsStackList = 'Stack Liste generee; %s ';
  RsModulesList = 'Liste des modules charges :';
  RsOSVersion = 'Systeme    : %s %s, Version: %d.%d, Build: %x, "%s"';
  RsProcessor = 'Processeur : %s, %s, %d MHz %s%s';
  RsScreenRes = 'Affichage  : %dx%d pixels, %d bpp';
  RsActiveControl = 'Hierarchie des controles actifs :';
  RsThread = 'Thread : %s';

type
  TExceptionForm = class(TForm)
    Panel1: TPanel;
    Image1: TImage;
    Panel2: TPanel;
    Panel3: TPanel;
    Panel4: TPanel;
    TerminateBTN: TButton;
    CopyBTN: TButton;
    CloseBTN: TButton;
    Panel5: TPanel;
    Label1: TLabel;
    Panel6: TPanel;
    ErrorLog: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure CloseBTNClick(Sender: TObject);
    procedure CopyBTNClick(Sender: TObject);
    procedure TerminateBTNClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure fillversion();
  end;

type TerrorHandler=class
    Private
    ExceptionShowing:boolean;
    product:string;
    _path:string;
    public
    procedure AppException(Sender: TObject; E: Exception);
    procedure AnyExceptionNotify(ExceptObj: TObject; ExceptAddr: Pointer; OSException: Boolean);
    procedure ShowForm(list:tstrings);
    procedure FillList(List:TStrings);
    property DataPath:string read _path write _path;
    constructor create;
end;
var
  ExceptionForm: TExceptionForm;
  errorHandler:TerrorHandler;
procedure InitErrorHandler(product:string='');
procedure CloseErrorHandler;

implementation
uses
   {$ifdef FullDebug}
   JclHookExcept,
   {$endif}
   JclDebug, rsFileVersion;
const SEP='----------------------------------------------------------------';
//==================================================================================================
// Helper routines
//==================================================================================================

function GetBPP: Integer;
var
  DC: HDC;
begin
  DC := GetDC(0);
  Result := GetDeviceCaps(DC, BITSPIXEL) * GetDeviceCaps(DC, PLANES);
  ReleaseDC(0, DC);
end;

//--------------------------------------------------------------------------------------------------

function SortModulesListByAddressCompare(List: TStringList; Index1, Index2: Integer): Integer;
begin Result := Integer(List.Objects[Index1]) - Integer(List.Objects[Index2]) end;

procedure InitErrorHandler(product:string='');
begin
ErrorHandler:=TErrorHandler.create;
errorHandler.product:=product;
end;

procedure CloseErrorHandler;
begin ErrorHandler.Free end;

constructor TerrorHandler.create;
begin
ExceptionShowing:=false;
JclStackTrackingOptions := JclStackTrackingOptions + [stRawMode];
_path :='';
{$ifndef FullDebug}
Application.OnException := AppException;
{$endif}
{$ifdef FullDebug}
  // Assign notification procedure for hooked RaiseException API call. This
  // allows being notified of any exception
  JclAddExceptNotifier(AnyExceptionNotify,npNormal);
{$endif}

end;

procedure TerrorHandler.FillList(List:TStrings);
const
  MMXText: array[Boolean] of PChar = ('', 'MMX');
  FDIVText: array[Boolean] of PChar = (' [FDIV Bug]', '');
var
  SL: TStringList;
  CpuInfo: TCpuInfo;
  StackList: TJclStackInfoList;
 {$IFDEF MODULES_LIST}
  I: Integer;
  ModuleName: TFileName;
  NtHeaders: PImageNtHeaders;
  ModuleBase: Cardinal;
  ImageBaseStr: string;
 {$ENDIF MODULES_LIST}
begin
SL := TStringList.Create;
try
  // Stack list
  StackList := JclLastExceptStackList;
  if Assigned(StackList) then
  begin
     List.Add(Format(RsStackList, [DateTimeToStr(StackList.TimeStamp)]));
     StackList.AddToStrings(List, true, True, True);
     List.Add(SEP);
  end;
  // System and OS information
  List.Add(Format(RsOSVersion, [GetWindowsVersionString, NtProductTypeString,
    Win32MajorVersion, Win32MinorVersion, Win32BuildNumber, Win32CSDVersion]));
  GetCpuInfo(CpuInfo);
  with CpuInfo do
    begin
    List.Add(Format(RsProcessor, [Manufacturer, CpuName,
      RoundFrequency(FrequencyInfo.NormFreq),
      MMXText[MMX], FDIVText[IsFDIVOK]]));
    List.Add(Format(RsScreenRes, [Screen.Width, Screen.Height, GetBPP]));
    List.Add(SEP);
  end;
 // Modules list
 {$IFDEF MODULES_LIST}
 if LoadedModulesList(SL, GetCurrentProcessId) then
 begin
    List.Add(RsModulesList);
    list.Add('');
    SL.CustomSort(SortModulesListByAddressCompare);
    for I := 0 to SL.Count - 1 do
    begin
      ModuleName := SL[I];
      ModuleBase := Cardinal(SL.Objects[I]);
      List.Add(Format('[%.8x] %s', [ModuleBase, ModuleName]));
      NtHeaders := PeMapImgNtHeaders(Pointer(ModuleBase));
      if (NtHeaders <> nil) and (NtHeaders^.OptionalHeader.ImageBase <> ModuleBase) then
         ImageBaseStr := Format('<%.8x> ', [NtHeaders^.OptionalHeader.ImageBase])
      else
         ImageBaseStr := '           ';

      List.Add(ImageBaseStr +' '+ getVersionStr(ModuleName));
    end;
  end;
  {$ENDIF MODULES_LIST}
finally
sl.free;
end;
end;

procedure TerrorHandler.AnyExceptionNotify(ExceptObj: TObject; ExceptAddr: Pointer; OSException: Boolean);
var
 list:tstringlist;
begin
  list:=tstringlist.create;
  try
  try
  list.clear;
  if exceptobj <> nil then
     list.Add(Exception(exceptobj).Message);

  list.add(_('Erreur : ')+inttostr(GetLastError)+':'+SysErrorMessage(GetLastError));
  list.Add(SEP);
  FillList(list);
  //JclLastExceptStackListToStrings(list, False, True, True);
  if OSException then list.Add('OSEXP');
  ShowForm(list);
  except
  end;
  finally
  list.Free;
end;
end;
{
//--------------------------------------------------------------------------------------------------

class procedure TExceptionDialog.ExceptionHandler(Sender: TObject; E: Exception);
begin
  if ExceptionShowing then
    Application.ShowException(E)
  else
  begin
    ExceptionShowing := True;
    try
      ShowException(E, nil);
    finally
      ExceptionShowing := False;
    end;
  end;
end;

//--------------------------------------------------------------------------------------------------

class procedure TExceptionDialog.ExceptionThreadHandler(Thread: TJclDebugThread);
begin
  if ExceptionShowing then
    Application.ShowException(Thread.SyncException)
  else
  begin
    ExceptionShowing := True;
    try
      ShowException(Thread.SyncException, Thread);
    finally
      ExceptionShowing := False;
    end;
  end;
end;

//--------------------------------------------------------------------------------------------------
}
{$ifndef FulldDebug}
procedure TerrorHandler.AppException(Sender: TObject; E: Exception);
var
 list:tstringlist;
begin
  list:=tstringlist.create;
  try
  try
  list.clear;
  list.Add(e.Message);
  if product > '' then list.Add(product);
  list.Add(SEP);
  //JclLastExceptStackListToStrings(list, False, True, True);
  FillList(list);
  showform(list);
  except
  end;
  finally
  list.free;
  end;
end;
{$endif}

procedure AppendToAFile(filename:string;list:tstrings);
var f:textfile;
    i:integer;
begin
try
   {$I-}
   if not fileexists(filename) then begin
      assignfile(f,filename);
      rewrite(f);
      closefile(f);
   end;
if fileexists(filename) then begin
   assignfile(f,filename);
   append(f);
   writeln(f,_('############# DEBUT DE L''ERREUR : ')+DateTimeToStr(now));
   writeln(f,application.exename);
   for i:=0 to list.Count -1 do
      writeln(f,list.strings[i]);
   writeln(f,_('### FIN DU JOURNAL'));
   writeln(f,'');
   writeln(f,'');
   closefile(f);
   {$I+}
end;
except
end;
end;

procedure TerrorHandler.ShowForm(list:tstrings);
var
 ef: TExceptionForm;
 dir:string;
begin
  ef:=TExceptionForm.Create(application);
  ef.ErrorLog.Lines.Assign(list);
  ef.fillversion;
  try
  if (_path <> '') and (DirectoryExists(_path)) then begin
     dir:=trim(_path);
     if dir[length(dir)]<>'\' then
        dir:=dir+'\';
     end
     else
     dir:=extractfilepath(application.exename);
  AppendToAFile(dir+'ExceptionsLog.txt',ef.ErrorLog.Lines);
  except
  end;
  MessageBeep(MB_ICONERROR);
  if ef.ShowModal=mrabort then begin
  try
    halt(1);
  except
  end;
  end;
end;


procedure TExceptionForm.fillversion();
var    sr:tsearchrec;
begin
ErrorLog.Lines.Add(SEP);
ErrorLog.Lines.Add(_('Versions - Produits'));
ErrorLog.Lines.Add('');
//version info
if (findfirst(extractfilepath(application.exename)+'*.exe',faAnyFile,sr)=0) then
   repeat
     ErrorLog.Lines.Add(sr.Name+':'+getVersionStr(extractfilepath(application.ExeName)+sr.Name)+' '+DateTimeToStr(FileDateToDateTime(sr.Time)));
   until findnext(sr)<>0;
if (findfirst(extractfilepath(application.exename)+'*.dll',faAnyFile,sr)=0) then
   repeat
     ErrorLog.Lines.Add(sr.Name+':'+getVersionStr(extractfilepath(application.ExeName)+sr.Name)+' '+DateTimeToStr(FileDateToDateTime(sr.Time)));
   until findnext(sr)<>0;
if (findfirst(extractfilepath(application.exename)+'*.bpl',faAnyFile,sr)=0) then
   repeat
     ErrorLog.Lines.Add(sr.Name+':'+getVersionStr(extractfilepath(application.ExeName)+sr.Name)+' '+DateTimeToStr(FileDateToDateTime(sr.Time)));
   until findnext(sr)<>0;
end;

{$R *.dfm}

procedure TExceptionForm.FormCreate(Sender: TObject);
begin
try
errorlog.Clear;
except

end;
end;


procedure TExceptionForm.CloseBTNClick(Sender: TObject);
begin
modalresult:=mrok;
end;

procedure TExceptionForm.CopyBTNClick(Sender: TObject);
begin
clipboard.AsText:=errorlog.Lines.Text ;
end;

procedure TExceptionForm.TerminateBTNClick(Sender: TObject);
begin
modalresult:=mrabort;
//Application.Terminate;
end;

initialization
  // Start Exception tracking
  JclStartExceptionTracking;
end.
